<?php

namespace Myriad\Authentication\Lib\Jobs;

use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Contracts\Bus\Dispatcher;
use Illuminate\Contracts\Validation\Factory;
use Illuminate\Validation\ValidationException;
use Myriad\Installations\InstallationManager;

class LoginWithEmailFromInstallation
{
    private $request;

    public function __construct($request)
    {
        $this->request = $request;
    }

    public static function getRules()
    {
        return [
            'email' => 'required|email',
            'password' => 'required|string',
            'installation_uuid' => 'required|string',
        ];
    }

    public static function validate(Factory $validationFactory, array $request)
    {
        // validate request
        $validator = $validationFactory->make($request, self::getRules());

        if ($validator->fails()) {
            throw new ValidationException($validator);
        }
    }

    public function handle(
        InstallationManager $installationManager,
        Factory $validationFactory,
        UserProvider $userProvider,
        Dispatcher $dispatcher
    ) {
        self::validate($validationFactory, $this->request);
        $user = $userProvider->retrieveByCredentials($this->request);

        $tokenRequest = [
            'user_id' => $user->getAuthIdentifier(),
            'installation_uuid' => $this->request['installation_uuid'],
        ];

        $accessToken = $dispatcher->dispatchNow(new CreateUserInstallationAccessToken($tokenRequest));

        return [
            'access_token' => $accessToken->uuid,
            'expires_at' => $accessToken->expires_at->toIso8601String(),
        ];
    }
}
