<?php

namespace Myriad\Authentication\Lib\Strategies;

use Illuminate\Contracts\Config\Repository as Config;
use Illuminate\Contracts\Events\Dispatcher;
use Illuminate\Contracts\Mail\Mailer;
use Myriad\Authentication\Lib\Events\EmailVerified;
use Myriad\Authentication\Lib\Repositories\IUserRepository;
use Myriad\Authentication\Lib\Services\IEmailConfirmer;
use Myriad\Authentication\Models\UserEmail;
use Myriad\Authentication\Models\VerificationCode;

class EmailVerification implements IVerificationStrategy, IEmailConfirmer
{
    use CreatesVerificationCodes;
    use VerifiesUserContactMethod;

    private $mailer;
    private $dispatcher;

    /**
     * @param  IUserRepository  $userRepository
     */
    public function __construct(
        IUserRepository $userRepository,
        Mailer $mailer,
        Config $config,
        Dispatcher $dispatcher
    ) {
        $this->userRepository = $userRepository;
        $this->mailer = $mailer;
        $this->dispatcher = $dispatcher;

        // set the properties necessary for VerifiesUserContactMethod
        $this->config = $config;
        $this->model = UserEmail::class;
        $this->property = "email";
        $this->contactCollection = "emailAddresses";
        $this->contactAlreadyExistsError = "Email address belongs to a different user!";
    }

    public function sendConfirmationEmail($email, $user, array $attributes)
    {
        $attributes = [
            'type' => array_key_exists('type', $attributes) ? $attributes['type'] : 'personal',
            'primary' => array_key_exists('primary', $attributes) ? $attributes['primary'] : false,
            'user_id' => is_null($user) ? null : $user->getAuthIdentifier(),
        ];

        $code = $this->createVerificationCode($email, EmailVerification::class, $attributes);

        $emailTemplate = $this->config->get('authentication.email.templates.email_address_verification');

        // send an email asking user to verify their code
        $this->mailer->send(
            $emailTemplate['template'],
            $this->getEmailData($code),
            function ($message) use ($email, $emailTemplate) {
                $message->to($email)->subject($emailTemplate['subject']);
            }
        );
    }

    /**
     * Raise an email verified event
     *
     * @param  UserEmail  $userEmail
     */
    public function raiseContactVerifiedEvent($userEmail)
    {
        $this->dispatcher->dispatch(new EmailVerified($userEmail));
    }

    protected function getEmailData(VerificationCode $code)
    {
        return [
            'code' => $code->code,
            'app_name' => $this->config->get('authentication.email.app_name'),
        ];
    }

}
