<?php

namespace Myriad\Authentication\Models;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\MorphTo;
use Illuminate\Database\Eloquent\SoftDeletes;
use Myriad\Installations\Models\Installation;

class AccessToken extends Model
{
    use SoftDeletes;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'token',
        'owner_type',
        'owner_id',
        'client_type',
        'client_id',
        'expires_at',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'token',
    ];

    /**
     * The attributes that should be mutated to dates.
     *
     * @var array
     */
    protected $dates = [
        'expires_at',
    ];

    /**
     * Get the owner object for this token.
     *
     * @return MorphTo|User
     */
    public function owner(): MorphTo
    {
        return $this->morphTo();
    }

    /**
     * Get the client object for this token.
     *
     * @return MorphTo|Installation
     */
    public function client(): MorphTo
    {
        return $this->morphTo();
    }

    /**
     * @return bool
     */
    public function isExpired(): bool
    {
        return now()->lt($this->expires_at);
    }

    /**
     * @param  Authenticatable  $user
     * @return bool
     */
    public function isForUser(Authenticatable $user): bool
    {
        return $this->owner->getAuthIdentifier() === $user->getAuthIdentifier();
    }
}
